@@ -1,5 +1,6 @@
 #include <qpdf/Pl_PNGFilter.hh>
 #include <qpdf/Pl_StdioFile.hh>
+#include <qpdf/QUtil.hh>
 
 #include <iostream>
 #include <errno.h>
@@ -7,23 +8,11 @@
 #include <string.h>
 #include <stdlib.h>
 
-FILE* safe_fopen(char const* filename, char const* mode)
-{
-    FILE* result = fopen(filename, mode);
-    if (result == 0)
-    {
-	std::cerr << "fopen " << filename << " failed: " << strerror(errno)
-		  << std::endl;
-	exit(2);
-    }
-    return result;
-}
-
 void run(char const* filename, bool encode, unsigned int columns)
 {
     // Decode the file
-    FILE* in = safe_fopen(filename, "rb");
-    FILE* o1 = safe_fopen("out", "wb");
+    FILE* in = QUtil::safe_fopen(filename, "rb");
+    FILE* o1 = QUtil::safe_fopen("out", "wb");
     Pipeline* out = new Pl_StdioFile("out", o1);
     Pipeline* pl = new Pl_PNGFilter(
 	"png", out,
@@ -72,7 +61,7 @@ int main(int argc, char* argv[])
     }
     bool encode = (strcmp(argv[1], "encode") == 0);
     char* filename = argv[2];
-    int columns = atoi(argv[3]);
+    int columns = QUtil::string_to_int(argv[3]);
 
     try
     {
